--- Main common api and loader
-- @author Oskar Eisemuth
-- @copyright 2017-2019
-- @module commonapi

--assert(commonapi ~= nil, "Common API init should be loaded once into lua")
local commonapi = {
--- Version of Common API as integer: YYYYMMDD
	version = 20190920,
	dmp = function(...)
		local ok, inspect = pcall(require, "commonapi.extlib.inspect")
		if (ok) then
			print(inspect(...))
		end
	end,
	_missing = {
		_native = true,
		ui = true,
	}
}


local commonapi_metatable = {
	__index = function(t, key)
		if (t._missing[key] ~= nil) then
			return nil
		end
		local ok, obj = pcall(require, "commonapi."..key)
		if (ok) then
			t[key] = obj
			return t[key]
		end
		print("commonapi."..key.." not found: "..obj)
		--print(debug.traceback())
		return nil
	end
}

setmetatable(commonapi, commonapi_metatable)


local function detectOS()
	local libext = package.cpath:match("%.[\\|/]?%?%.(%a+)")
	local mypath = debug.getinfo(2, "S").source:sub(2)

	local libexttosys = {
		dll = 1,
		so = 2,
		dylib = 3
	}

	mypath = string.gsub(mypath, '\\', '/')
	mypath = string.gsub(mypath, "/res/scripts/commonapi/init%.lua$", "/")

	local info = {
		sys = libexttosys[libext] or 0,
		libext = libext,
		path = mypath,
		settings = {}
	}

	commonapi._info = info
end


local function loadSettings()
	local result, luafnSettings, err
	local filename = commonapi._info.path.."settings.lua"
	local env = {}
	setmetatable(env, {__index=_G})
	luafnSettings, err = loadfile(filename, "bt", env)

	if (luafnSettings == nil) then
		return
	end

	result = luafnSettings()
	if (type(result) ~= "table") then
		print("commonapi.init: Result of "..filename.. " not a table?!")
	end
	commonapi._info.settings = result
end

local function loadCommonAPILuaExtension()
--	local mypath = debug.getinfo(2, "S").source:sub(2)
	local sys = commonapi._info.sys
	
	if not ( sys == 1 or sys == 2 ) then
		return
	end

--	mypath = string.gsub(mypath, '\\', '/')
--	mypath = string.gsub(mypath, "/res/scripts/commonapi/init%.lua$", "/bin/?."..commonapi.osinfo.libext)

	local mypath = commonapi._info.path.."bin/?."..commonapi._info.libext
	local oldcpath = package.cpath
	local oldpath = package.path
	package.cpath = mypath
	package.path = ""

	commonapi._missing['_native'] = true
	commonapi._missing['ui'] = true
	local ok, dll = pcall(require, "CommonAPILua")
	package.cpath = oldcpath
	package.path = oldpath

	if not ok then
		print("commonapi.init: Can't load native code CommonAPILua: "..dll)
		return
	elseif (type(dll) == 'table') then
		commonapi._native = dll
		if (type(commonapi._info.settings.verbosity) == "number") then
			commonapi._native.setDebugVerbosity(commonapi._info.settings.verbosity)
		end
		commonapi._native.setSettings(commonapi._info.settings)

		commonapi._native.doPatches()

		commonapi.ui = commonapi._nativeui
		commonapi.ui._activate()

--		if (type(commonapi._info.settings.uiscale) == "number") then
--			commonapi._native.setUIScale(commonapi._info.settings.uiscale)
--		end

		if (commonapi._events ~= nil) then
			if (commonapi._events._activate()) then
				commonapi.events = commonapi._events
			end
		end

		if (commonapi._vehanimations ~= nil) then
			if (commonapi._vehanimations._activate()) then
				commonapi.vehanimations = commonapi._vehanimations
			end
		end
	end
end

local function loadTranslationSystem(translations)
	if (setStrings == nil) then
		require "mod"
	end
	setStrings("eis_os_commonapi_1", translations)

	if (getTextRes == nil) then
		getTextRes = function(s)
			return s
		end
	end

	local lang = "en"

	if (commonapi._native ~= nil) then
		lang = commonapi._native.getLanguage()
	end

	if (_ == nil) then
		local translatefn = 'function _(s) return translate("eis_os_commonapi_1", "'..lang..'", s) end'
		load(translatefn)()
	end

	if (commonapi._native ~= nil) then
		if (translations[lang] ~= nil) then
			commonapi._native.setTranslations(translations[lang])
		else
			commonapi._native.setTranslations(translations["en"])
		end
	end
end

function commonapi._init(mystrings)
	if (_G['commonapi'] ~= nil) then
-- CommonAPI strings.lua could be loaded again in the same context
--		print("ERROR: Common API was loaded already? You can only have one commonapi installed!")
		return
	end
	print("commonapi.init", commonapi.version)
	_G['commonapi'] = commonapi

	detectOS()
	loadSettings()


	if (commonapi._info.settings.disablenative == true) then
		print("commonapi.init: Native code loading disabled by settings.lua")
	else
		loadCommonAPILuaExtension()
	end
	loadTranslationSystem(mystrings)

	if (commonapi.ui ~= nil) then
		local modsettings = require "commonapi.ui.modsettings"
		modsettings.run()
		local uimisc = require "commonapi.ui.misc"
	end
	-- Need to load early for proper hooking...
	local mods = commonapi.mods

end


return commonapi